//******************************************************************************
//  FILE..........: CASLINK2.H
//  COPYRIGHT.....: Copyright (C) 1999-2008 Alexey Podrezov
//  VERSION.......: 2.1
//  DESCRIPTION...: Cassette Interface Emulation utility for MSX computers
//  NOTES.........: Encodes/decodes files to be transfered through MSX cassette interface
//  HISTORY.......:
//  PERSON          DATE         COMMENT
//  --------        ----------   -----------------------------------------------
//  Podrezov        07/09/1999   Initial version created
//  Podrezov        26/08/2005   Sources ported to Visual Studio .NET
//  Podrezov        28/08/2005   Added support for 16 bit, stereo, 44 kHz
//  Podrezov        29/08/2005   Modified ROM loaders
//  Podrezov        30/08/2005   Added sound file playback
//  Podrezov        05/09/2005   Increased parts number to 15, multi-file loader modified
//  Podrezov        12/09/2005   Fixed detokenized Basic file loading bug in multi-file loader routine
//  Podrezov        25/09/2005   Added preloader for BINs, ROMs
//  Podrezov        26/09/2005   Modified and recompiled ROM loaders, created BIN2INC utility
//  Podrezov        29/09/2005   Added preloader for multi-file ROMs
//  Podrezov        03/10/2005   Wrote WAV converter from CD to RD quality
//  Podrezov        14/03/2008   Sources ported to Visual Studio 2005
//  Podrezov        17/03/2008   Added MP3 conversion capability, added colors for messages
//  Podrezov        19/03/2008   Disabled signal level question (now it always is 100), fixed waveform of CD quality WAV file
//  Podrezov        21/03/2008   Added option for normal and inverted waveforms for different MP3 players
//  Podrezov        22/03/2008   Fixed a bug in WAV file length calculation when only binary files are encoded
//  Podrezov        23/03/2008   Added copyright message to WAV files, added wrong CR+LF check for input ASCII files
//  Podrezov        26/03/2008   Re-wrote all ROM loaders, created single BIN file loader, added CRC check
//  Podrezov        28/03/2008   Added 3600 baud MSX cassette I/O speed support!
//  Podrezov        05/08/2008   Added missing detokenized file encoding code in single file encoding routine
//  Podrezov        29/10/2008   Fixed a bug in 32kb ROM loading routine, now all such ROMs should start normally
//******************************************************************************

// Defaults
#define	STARTBIT				0
#define	STOPBIT					1
#define	SILENCE					0x80
#define	SILENCE16				0
#define HIGHAMPLITUDE			0xFF
#define LOWAMPLITUDE			0
#define HIGHAMPLITUDE16			0x7FFF
#define LOWAMPLITUDE16			0x8000
#define TOLERANCELEVEL			0x10
#define TOLERANCELEVEL16		0x800
#define CASFILENAMELENGTH		6
#define WAVHEADEROFFSETVALUE	0x77
#define MINWAVINPUTFILELENGTH	1024L
#define MINENCINPUTFILELENGTH	5L
#define MAXENCINPUTFILELENGTH	33000L
#define LENGTHCORRECTION		25
#define USERBUFFERLENGTH		1024
#define MAXSIGNALLEVEL			100

// Variables -------------------------------------------------------------------
HANDLE	hStdIn, hStdOut;
FILE	*InputFileHandle, *OutputFileHandle, *TmpFileHandle, *BatFileHandle;
char	InputFileName[30], OutputFileName[30], TmpFileName[30], BatFileName[30], MP3FileName[30];
char*	FileNamePointer[20];
unsigned char	BitStream[11], *InputMemPointer, *OutputMemPointer, FirstPauseLength, MiddlePauseLength, FinalPauseLength, SilenceByte, EncodeDecode, FileType, AddressBuffer[6], RomCRC, ConvertRatio;
unsigned int	WavSampleRate, SignalLevel, MSXInterfaceSpeed, LongHeaderLength, ShortHeaderLength, SampleScale, SpeedScale, k, l, ConvertFlag, ConvertToMP3, MP3Kbps, InvertedWaveform;
unsigned short	RomBegin, RomEnd, RomStart, *InputMemPointer16;
size_t	InputFileLength, OutputFileLength, TmpFileLength, BytesRead, BytesWritten, FileOffset, ExtraBytes, InputFileLengthTmp;
size_t	i, a, b;
DWORD	j, MoreExtraBytes, Result;
static char szLameEXE[]="lame.exe";
static char szLameDLL[]="lame_enc.dll";

// Wave header
struct {
	char	WavHeaderID[4];		// 'RIFF'
	DWORD	SampleLength;
	char	MMFileType[4];		// 'WAVE'
	char	FormatID[4];		// 'fmt '
	DWORD	FormatSize;
	WORD	FormatTag;
	WORD	NumChannels;
	DWORD	SamplesPerSec;
	DWORD	BytesPerSec;
	WORD	BlkAllign;
	WORD	BitsPerSample;
	char	DataID[4];			// 'data'
	DWORD	PureSampleLength;
	} WavHeader;

// Headers
static unsigned char	BasicFileHeaderID[10] = {0xd3, 0xd3, 0xd3, 0xd3, 0xd3, 0xd3, 0xd3, 0xd3, 0xd3, 0xd3};
static unsigned char	BinFileHeaderID[10] = {0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0};
static unsigned char	DataFileHeaderID[10] = {0xea, 0xea, 0xea, 0xea, 0xea, 0xea, 0xea, 0xea, 0xea, 0xea};
unsigned char	BitEncodingLength, FileHeaderIDLength;
static char		NameBuffer[CASFILENAMELENGTH], Buffer[USERBUFFERLENGTH], ch;

// Patterns of 1200Hz and 2400Hz waves - samples for 11025Hz file. if bigger sampling rate is used, multiply by SampleScale variable
static unsigned char	ZeroBit[2][10] = {
	{LOWAMPLITUDE, LOWAMPLITUDE, LOWAMPLITUDE, LOWAMPLITUDE, LOWAMPLITUDE, HIGHAMPLITUDE, HIGHAMPLITUDE, HIGHAMPLITUDE, HIGHAMPLITUDE, HIGHAMPLITUDE},
	{LOWAMPLITUDE, LOWAMPLITUDE, HIGHAMPLITUDE, HIGHAMPLITUDE, HIGHAMPLITUDE, LOWAMPLITUDE, LOWAMPLITUDE, HIGHAMPLITUDE, HIGHAMPLITUDE, HIGHAMPLITUDE}
	};
static unsigned char	SetBit[2][10] = {
	{LOWAMPLITUDE, LOWAMPLITUDE, HIGHAMPLITUDE, HIGHAMPLITUDE, HIGHAMPLITUDE, LOWAMPLITUDE, LOWAMPLITUDE, HIGHAMPLITUDE, HIGHAMPLITUDE, HIGHAMPLITUDE},
	{LOWAMPLITUDE, HIGHAMPLITUDE, LOWAMPLITUDE, HIGHAMPLITUDE, LOWAMPLITUDE, HIGHAMPLITUDE, LOWAMPLITUDE, HIGHAMPLITUDE, LOWAMPLITUDE, HIGHAMPLITUDE}
	};

// BIN/ROM loaders and preloader
static unsigned char	BINLoader[206] = {
	0xC3, 0x30, 0x90, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x5B, 0x43, 0x61, 0x73, 0x4C, 0x69,
	0x6E, 0x6B, 0x32, 0x5D, 0x20, 0x4C, 0x6F, 0x61, 0x64, 0x69, 0x6E, 0x67, 0x20, 0x66, 0x61, 0x69,
	0x6C, 0x65, 0x64, 0x3A, 0x20, 0x43, 0x52, 0x43, 0x20, 0x45, 0x52, 0x52, 0x4F, 0x52, 0x21, 0x00,
	0xF3, 0x2A, 0x03, 0x90, 0xED, 0x5B, 0x05, 0x90, 0xEB, 0xED, 0x52, 0xE5, 0xC1, 0x21, 0xCE, 0x90,
	0xAF, 0xF5, 0xF1, 0x86, 0x23, 0x0B, 0xF5, 0x79, 0xB7, 0xC2, 0x42, 0x90, 0x78, 0xB7, 0xC2, 0x42,
	0x90, 0xF1, 0x47, 0x21, 0x09, 0x90, 0x7E, 0xB8, 0xCA, 0x9C, 0x90, 0xFB, 0xCD, 0x6C, 0x00, 0x3E,
	0x0F, 0x21, 0xE9, 0xF3, 0x77, 0x3E, 0x08, 0x23, 0x77, 0x23, 0x77, 0xCD, 0x62, 0x00, 0xAF, 0xCD,
	0xC3, 0x00, 0xCD, 0xCF, 0x00, 0x21, 0x01, 0x01, 0xCD, 0xC6, 0x00, 0x11, 0x0A, 0x90, 0x1A, 0xB7,
	0xCA, 0x8E, 0x90, 0x13, 0xCD, 0xA2, 0x00, 0x24, 0xCD, 0xC6, 0x00, 0xC3, 0x7E, 0x90, 0x21, 0x03,
	0x01, 0xCD, 0xC6, 0x00, 0xCD, 0xC0, 0x00, 0xCD, 0x56, 0x01, 0xFB, 0xC9, 0xF3, 0x21, 0xB2, 0x90,
	0x11, 0xCF, 0x90, 0xEB, 0xED, 0x52, 0x44, 0x4D, 0x21, 0xB2, 0x90, 0x11, 0x60, 0xF5, 0xD5, 0xED,
	0xB0, 0xC9, 0x2A, 0x07, 0x90, 0xE5, 0x2A, 0x03, 0x90, 0xED, 0x5B, 0x05, 0x90, 0xEB, 0xED, 0x52,
	0x44, 0x4D, 0x21, 0xCE, 0x90, 0xED, 0x5B, 0x03, 0x90, 0xED, 0xB0, 0xFB, 0xC9, 0x00
	};

static unsigned char	ROMLoader[227] = {
	0xC3, 0x30, 0x90, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x5B, 0x43, 0x61, 0x73, 0x4C, 0x69,
	0x6E, 0x6B, 0x32, 0x5D, 0x20, 0x4C, 0x6F, 0x61, 0x64, 0x69, 0x6E, 0x67, 0x20, 0x66, 0x61, 0x69,
	0x6C, 0x65, 0x64, 0x3A, 0x20, 0x43, 0x52, 0x43, 0x20, 0x45, 0x52, 0x52, 0x4F, 0x52, 0x21, 0x00,
	0xF3, 0x2A, 0x03, 0x90, 0xED, 0x5B, 0x05, 0x90, 0xEB, 0xED, 0x52, 0xE5, 0xC1, 0x21, 0xE3, 0x90,
	0xAF, 0xF5, 0xF1, 0x86, 0x23, 0x0B, 0xF5, 0x79, 0xB7, 0xC2, 0x42, 0x90, 0x78, 0xB7, 0xC2, 0x42,
	0x90, 0xF1, 0x47, 0x21, 0x09, 0x90, 0x7E, 0xB8, 0xCA, 0x9C, 0x90, 0xFB, 0xCD, 0x6C, 0x00, 0x3E,
	0x0F, 0x21, 0xE9, 0xF3, 0x77, 0x3E, 0x08, 0x23, 0x77, 0x23, 0x77, 0xCD, 0x62, 0x00, 0xAF, 0xCD,
	0xC3, 0x00, 0xCD, 0xCF, 0x00, 0x21, 0x01, 0x01, 0xCD, 0xC6, 0x00, 0x11, 0x0A, 0x90, 0x1A, 0xB7,
	0xCA, 0x8E, 0x90, 0x13, 0xCD, 0xA2, 0x00, 0x24, 0xCD, 0xC6, 0x00, 0xC3, 0x7E, 0x90, 0x21, 0x03,
	0x01, 0xCD, 0xC6, 0x00, 0xCD, 0xC0, 0x00, 0xCD, 0x56, 0x01, 0xFB, 0xC9, 0xF3, 0x2A, 0x07, 0x90,
	0xE5, 0x7C, 0xE6, 0xC0, 0x11, 0x00, 0x00, 0x21, 0x0F, 0xF0, 0x28, 0x06, 0x11, 0x00, 0x40, 0x21,
	0x0C, 0xF3, 0x3A, 0xFF, 0xFF, 0x2F, 0xA4, 0x4F, 0x0F, 0x0F, 0x0F, 0x0F, 0xA5, 0xB1, 0x32, 0xFF,
	0xFF, 0xDB, 0xA8, 0xA4, 0x47, 0x0F, 0x0F, 0x0F, 0x0F, 0xA5, 0xB0, 0xD3, 0xA8, 0xD5, 0x2A, 0x03,
	0x90, 0xED, 0x5B, 0x05, 0x90, 0xEB, 0xED, 0x52, 0x44, 0x4D, 0x21, 0xE3, 0x90, 0xD1, 0xED, 0xB0,
	0xFB, 0xC9, 0x00
	};

static unsigned char	ROM32KLoader1[378] = {
	0xC3, 0x76, 0x90, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1E, 0x62, 0x6C, 0x6F, 0x61, 0x64,
	0x22, 0x63, 0x61, 0x73, 0x3A, 0x62, 0x69, 0x6E, 0x72, 0x6F, 0x6D, 0x20, 0x70, 0x61, 0x72, 0x74,
	0x20, 0x32, 0x20, 0x22, 0x2C, 0x72, 0x0D, 0x00, 0x5B, 0x43, 0x61, 0x73, 0x4C, 0x69, 0x6E, 0x6B,
	0x32, 0x5D, 0x20, 0x4C, 0x6F, 0x61, 0x64, 0x69, 0x6E, 0x67, 0x20, 0x66, 0x69, 0x6C, 0x65, 0x2C,
	0x20, 0x70, 0x6C, 0x65, 0x61, 0x73, 0x65, 0x20, 0x77, 0x61, 0x69, 0x74, 0x2E, 0x2E, 0x2E, 0x00,
	0x5B, 0x43, 0x61, 0x73, 0x4C, 0x69, 0x6E, 0x6B, 0x32, 0x5D, 0x20, 0x4C, 0x6F, 0x61, 0x64, 0x69,
	0x6E, 0x67, 0x20, 0x66, 0x61, 0x69, 0x6C, 0x65, 0x64, 0x3A, 0x20, 0x43, 0x52, 0x43, 0x20, 0x45,
	0x52, 0x52, 0x4F, 0x52, 0x21, 0x00, 0xF3, 0x2A, 0x03, 0x90, 0xED, 0x5B, 0x05, 0x90, 0xEB, 0xED,
	0x52, 0xE5, 0xC1, 0x21, 0x7A, 0x91, 0xAF, 0xF5, 0xF1, 0x86, 0x23, 0x0B, 0xF5, 0x79, 0xB7, 0xC2,
	0x88, 0x90, 0x78, 0xB7, 0xC2, 0x88, 0x90, 0xF1, 0x47, 0x21, 0x09, 0x90, 0x7E, 0xB8, 0xCA, 0xE2,
	0x90, 0xFB, 0xCD, 0x6C, 0x00, 0x3E, 0x0F, 0x21, 0xE9, 0xF3, 0x77, 0x3E, 0x08, 0x23, 0x77, 0x23,
	0x77, 0xCD, 0x62, 0x00, 0xAF, 0xCD, 0xC3, 0x00, 0xCD, 0xCF, 0x00, 0x21, 0x01, 0x01, 0xCD, 0xC6,
	0x00, 0x11, 0x50, 0x90, 0x1A, 0xB7, 0xCA, 0xD4, 0x90, 0x13, 0xCD, 0xA2, 0x00, 0x24, 0xCD, 0xC6,
	0x00, 0xC3, 0xC4, 0x90, 0x21, 0x03, 0x01, 0xCD, 0xC6, 0x00, 0xCD, 0xC0, 0x00, 0xCD, 0x56, 0x01,
	0xFB, 0xC9, 0xFB, 0xCD, 0x6C, 0x00, 0x3E, 0x0F, 0x21, 0xE9, 0xF3, 0x77, 0x3E, 0x04, 0x23, 0x77,
	0x23, 0x77, 0xCD, 0x62, 0x00, 0xAF, 0xCD, 0xC3, 0x00, 0xCD, 0xCC, 0x00, 0x21, 0x01, 0x01, 0xCD,
	0xC6, 0x00, 0x11, 0x28, 0x90, 0x1A, 0xB7, 0xCA, 0x15, 0x91, 0x13, 0xCD, 0xA2, 0x00, 0x24, 0xCD,
	0xC6, 0x00, 0xC3, 0x05, 0x91, 0x21, 0x03, 0x01, 0xCD, 0xC6, 0x00, 0xCD, 0x56, 0x01, 0xF3, 0x21,
	0xF0, 0xFB, 0x22, 0xF8, 0xF3, 0x22, 0xFA, 0xF3, 0x21, 0x0A, 0x90, 0x11, 0xF0, 0xFB, 0x01, 0x1D,
	0x00, 0xED, 0xB0, 0x21, 0x0D, 0xFC, 0x22, 0xF8, 0xF3, 0xF3, 0x3A, 0xFF, 0xFF, 0x2F, 0xF5, 0x4F,
	0xE6, 0xF0, 0x47, 0x79, 0x0F, 0x0F, 0x0F, 0x0F, 0xE6, 0x0F, 0xB0, 0x32, 0xFF, 0xFF, 0xDB, 0xA8,
	0xF5, 0xE6, 0xF0, 0x47, 0x0F, 0x0F, 0x0F, 0x0F, 0xB0, 0xD3, 0xA8, 0x2A, 0x03, 0x90, 0xED, 0x5B,
	0x05, 0x90, 0xEB, 0xED, 0x52, 0x44, 0x4D, 0x21, 0x7A, 0x91, 0xED, 0x5B, 0x03, 0x90, 0xED, 0xB0,
	0xF1, 0xD3, 0xA8, 0xF1, 0x32, 0xFF, 0xFF, 0xFB, 0xC9, 0x00
	};

static unsigned char	ROM32KLoader2[254] = {
	0xC3, 0x30, 0x90, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x5B, 0x43, 0x61, 0x73, 0x4C, 0x69,
	0x6E, 0x6B, 0x32, 0x5D, 0x20, 0x4C, 0x6F, 0x61, 0x64, 0x69, 0x6E, 0x67, 0x20, 0x66, 0x61, 0x69,
	0x6C, 0x65, 0x64, 0x3A, 0x20, 0x43, 0x52, 0x43, 0x20, 0x45, 0x52, 0x52, 0x4F, 0x52, 0x21, 0x00,
	0xF3, 0x2A, 0x03, 0x90, 0xED, 0x5B, 0x05, 0x90, 0xEB, 0xED, 0x52, 0xE5, 0xC1, 0x21, 0xFE, 0x90,
	0xAF, 0xF5, 0xF1, 0x86, 0x23, 0x0B, 0xF5, 0x79, 0xB7, 0xC2, 0x42, 0x90, 0x78, 0xB7, 0xC2, 0x42,
	0x90, 0xF1, 0x47, 0x21, 0x09, 0x90, 0x7E, 0xB8, 0xCA, 0x9C, 0x90, 0xFB, 0xCD, 0x6C, 0x00, 0x3E,
	0x0F, 0x21, 0xE9, 0xF3, 0x77, 0x3E, 0x08, 0x23, 0x77, 0x23, 0x77, 0xCD, 0x62, 0x00, 0xAF, 0xCD,
	0xC3, 0x00, 0xCD, 0xCF, 0x00, 0x21, 0x01, 0x01, 0xCD, 0xC6, 0x00, 0x11, 0x0A, 0x90, 0x1A, 0xB7,
	0xCA, 0x8E, 0x90, 0x13, 0xCD, 0xA2, 0x00, 0x24, 0xCD, 0xC6, 0x00, 0xC3, 0x7E, 0x90, 0x21, 0x03,
	0x01, 0xCD, 0xC6, 0x00, 0xCD, 0xC0, 0x00, 0xCD, 0x56, 0x01, 0xFB, 0xC9, 0x3A, 0xFF, 0xFF, 0x2F,
	0xF5, 0x4F, 0xE6, 0xF0, 0x47, 0x79, 0x0F, 0x0F, 0x0F, 0x0F, 0xE6, 0x0F, 0xB0, 0x32, 0xFF, 0xFF,
	0xDB, 0xA8, 0xF5, 0xE6, 0xF0, 0x47, 0x0F, 0x0F, 0x0F, 0x0F, 0xB0, 0xD3, 0xA8, 0x2A, 0x03, 0x90,
	0x7C, 0xE6, 0xF0, 0xFE, 0x80, 0xC2, 0xDD, 0x90, 0x21, 0xDD, 0x90, 0x11, 0xFF, 0x90, 0xEB, 0xED,
	0x52, 0x44, 0x4D, 0x21, 0xDD, 0x90, 0x11, 0x60, 0xF5, 0xD5, 0xED, 0xB0, 0xC9, 0x2A, 0x03, 0x90,
	0xED, 0x5B, 0x05, 0x90, 0xEB, 0xED, 0x52, 0x44, 0x4D, 0x21, 0xFE, 0x90, 0xED, 0x5B, 0x03, 0x90,
	0xED, 0xB0, 0xF1, 0xD3, 0xA8, 0xF1, 0x32, 0xFF, 0xFF, 0xF7, 0x00, 0x00, 0x00, 0x00
	};

static unsigned char	MultFileLoader[367] = {
	0xC3, 0x76, 0x90, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1E, 0x62, 0x6C, 0x6F, 0x61, 0x64,
	0x22, 0x63, 0x61, 0x73, 0x3A, 0x62, 0x69, 0x6E, 0x72, 0x6F, 0x6D, 0x20, 0x70, 0x61, 0x72, 0x74,
	0x20, 0x31, 0x20, 0x22, 0x2C, 0x72, 0x0D, 0x00, 0x5B, 0x43, 0x61, 0x73, 0x4C, 0x69, 0x6E, 0x6B,
	0x32, 0x5D, 0x20, 0x4C, 0x6F, 0x61, 0x64, 0x69, 0x6E, 0x67, 0x20, 0x66, 0x69, 0x6C, 0x65, 0x2C,
	0x20, 0x70, 0x6C, 0x65, 0x61, 0x73, 0x65, 0x20, 0x77, 0x61, 0x69, 0x74, 0x2E, 0x2E, 0x2E, 0x00,
	0x5B, 0x43, 0x61, 0x73, 0x4C, 0x69, 0x6E, 0x6B, 0x32, 0x5D, 0x20, 0x4C, 0x6F, 0x61, 0x64, 0x69,
	0x6E, 0x67, 0x20, 0x66, 0x61, 0x69, 0x6C, 0x65, 0x64, 0x3A, 0x20, 0x43, 0x52, 0x43, 0x20, 0x45,
	0x52, 0x52, 0x4F, 0x52, 0x21, 0x00, 0xF3, 0x2A, 0x03, 0x90, 0xED, 0x5B, 0x05, 0x90, 0xEB, 0xED,
	0x52, 0xE5, 0xC1, 0x21, 0x6F, 0x91, 0xAF, 0xF5, 0xF1, 0x86, 0x23, 0x0B, 0xF5, 0x79, 0xB7, 0xC2,
	0x88, 0x90, 0x78, 0xB7, 0xC2, 0x88, 0x90, 0xF1, 0x47, 0x21, 0x09, 0x90, 0x7E, 0xB8, 0xCA, 0xE2,
	0x90, 0xFB, 0xCD, 0x6C, 0x00, 0x3E, 0x0F, 0x21, 0xE9, 0xF3, 0x77, 0x3E, 0x08, 0x23, 0x77, 0x23,
	0x77, 0xCD, 0x62, 0x00, 0xAF, 0xCD, 0xC3, 0x00, 0xCD, 0xCF, 0x00, 0x21, 0x01, 0x01, 0xCD, 0xC6,
	0x00, 0x11, 0x50, 0x90, 0x1A, 0xB7, 0xCA, 0xD4, 0x90, 0x13, 0xCD, 0xA2, 0x00, 0x24, 0xCD, 0xC6,
	0x00, 0xC3, 0xC4, 0x90, 0x21, 0x03, 0x01, 0xCD, 0xC6, 0x00, 0xCD, 0xC0, 0x00, 0xCD, 0x56, 0x01,
	0xFB, 0xC9, 0x00, 0x00, 0xFB, 0xCD, 0x6C, 0x00, 0x3E, 0x0F, 0x21, 0xE9, 0xF3, 0x77, 0x3E, 0x04,
	0x23, 0x77, 0x23, 0x77, 0xCD, 0x62, 0x00, 0xAF, 0xCD, 0xC3, 0x00, 0xCD, 0xCC, 0x00, 0x21, 0x01,
	0x01, 0xCD, 0xC6, 0x00, 0x11, 0x28, 0x90, 0x1A, 0xB7, 0xCA, 0x17, 0x91, 0x13, 0xCD, 0xA2, 0x00,
	0x24, 0xCD, 0xC6, 0x00, 0xC3, 0x07, 0x91, 0x21, 0x03, 0x01, 0xCD, 0xC6, 0x00, 0xCD, 0x56, 0x01,
	0xF3, 0x21, 0xF0, 0xFB, 0x22, 0xF8, 0xF3, 0x22, 0xFA, 0xF3, 0x21, 0x0A, 0x90, 0x11, 0xF0, 0xFB,
	0x01, 0x1D, 0x00, 0xED, 0xB0, 0x21, 0x0D, 0xFC, 0x22, 0xF8, 0xF3, 0x00, 0x00, 0xF3, 0x21, 0x53,
	0x91, 0x11, 0x70, 0x91, 0xEB, 0xED, 0x52, 0x44, 0x4D, 0x21, 0x53, 0x91, 0x11, 0x60, 0xF5, 0xD5,
	0xED, 0xB0, 0xC9, 0x2A, 0x07, 0x90, 0xE5, 0x2A, 0x03, 0x90, 0xED, 0x5B, 0x05, 0x90, 0xEB, 0xED,
	0x52, 0x44, 0x4D, 0x21, 0x6F, 0x91, 0xED, 0x5B, 0x03, 0x90, 0xED, 0xB0, 0xFB, 0xC9, 0x00
	};

static unsigned char	BINROMPreLoader[163] = {
	0xC3, 0x49, 0x90, 0x1E, 0x62, 0x6C, 0x6F, 0x61, 0x64, 0x22, 0x63, 0x61, 0x73, 0x3A, 0x62, 0x69,
	0x6E, 0x72, 0x6F, 0x6D, 0x20, 0x70, 0x61, 0x72, 0x74, 0x20, 0x31, 0x20, 0x22, 0x2C, 0x72, 0x0D,
	0x00, 0x5B, 0x43, 0x61, 0x73, 0x4C, 0x69, 0x6E, 0x6B, 0x32, 0x5D, 0x20, 0x4C, 0x6F, 0x61, 0x64,
	0x69, 0x6E, 0x67, 0x20, 0x66, 0x69, 0x6C, 0x65, 0x2C, 0x20, 0x70, 0x6C, 0x65, 0x61, 0x73, 0x65,
	0x20, 0x77, 0x61, 0x69, 0x74, 0x2E, 0x2E, 0x2E, 0x00, 0xFB, 0xCD, 0x6C, 0x00, 0x3E, 0x0F, 0x21,
	0xE9, 0xF3, 0x77, 0x3E, 0x04, 0x23, 0x77, 0x23, 0x77, 0xCD, 0x62, 0x00, 0xAF, 0xCD, 0xC3, 0x00,
	0xCD, 0xCC, 0x00, 0x21, 0x01, 0x01, 0xCD, 0xC6, 0x00, 0x11, 0x21, 0x90, 0x1A, 0xB7, 0xCA, 0x7C,
	0x90, 0x13, 0xCD, 0xA2, 0x00, 0x24, 0xCD, 0xC6, 0x00, 0xC3, 0x6C, 0x90, 0x21, 0x03, 0x01, 0xCD,
	0xC6, 0x00, 0xCD, 0x56, 0x01, 0xF3, 0x21, 0xF0, 0xFB, 0x22, 0xF8, 0xF3, 0x22, 0xFA, 0xF3, 0x21,
	0x03, 0x90, 0x11, 0xF0, 0xFB, 0x01, 0x1D, 0x00, 0xED, 0xB0, 0x21, 0x0D, 0xFC, 0x22, 0xF8, 0xF3,
	0xFB, 0xC9, 0x00
	};

// Messages
static char	szCaption[]="CasLink2 Project. Created by Alexey Podrezov, 1999-2008";
static char	szTitle1[]="\nCasLink Version 2.1\nCopyright (c) 1999-2008 Alexey Podrezov. All rights reserved.\n";
static char szTitle2[]="\n This utility encodes/decodes files that are transfered via the MSX cassette\n interface with the help of most common sound input/output capable hardware:\n PC sound card, CD or MP3 player.\n\n Supported formats: BAS (tokenized/detokenized), SCR (screenshots)\n                    BIN (8/16/24/32 kb), ROM (8/16/24/32 kb)\n";

static char	FileFormat[][50] = {
	"BAS (tokenized Basic program)",
	"BIN (binary program)",
	"ROM (cartridge image)",
	"ASC (detokenized Basic program)",
	"unknown format"
	};

static char szHelp[]=		"\n Usage: CasLink2 <key> <file_name>\n\n  Keys:  /E  - encode Basic or binary file into WAV and MP3 sound files\n         /M  - encode multiple files (up to 15) into WAV and MP3 sound files\n         /D  - decode Basic or binary file from WAV sound file\n         /?  - this help screen\n\n  file_name  - name(s) of input file(s), maximum 12 characters (filename.ext)\n\n  Examples:\n            CasLink2 /e File1.rom\n            CasLink2 /m File1.rom File2.rom\n            CasLink2 /m File1.bas File2.rom File3.rom\n            CasLink2 /d File1.wav\n";
static char szNoAction[]=	"\n To get help on command line options use /? key...\n\n";
static char szEncSuccMsg1[]="\n The output WAV file was created successfully!\n Do you want to playback the created WAV file now? [Y/N] - ";
static char szEncSuccMsg2[]="\n The output files were created successfully!\n Do you want to playback the created WAV file now? [Y/N] - ";
static char szDecSuccMsg[]=	"\n All files have been successfully decoded!\n";
static char	szPressKey[]=	"\n Playback started... Press any key to stop playback and return to OS...\n";
static char szFinalMsg[]=	"\n Have a nice day!\n Please send your feedback to CasLink2 author:  alexey.podrezov@gmail.com\n\n";
static char szSpeed[]=		"\n  Select MSX cassette I/O speed: [1] 1200 baud, [2] 2400 baud, [3] 3600 baud - ";
static char szQuality[]=	"\n  Select WAV sampling rate: [1] 8 bit/mono/11 kHz; [2] 16 bit/stereo/44 kHz - ";
static char szInvertedWF[]=	"\n  Select waveform type: [1] Normal (Creative), [2] Inverted (iPOD, CD, PC) - ";
static char szConvert[]=	"\n  Would you also like to get MP3 file in addition to WAV [Y/N] - ";
static char szMP3Kbps[]=	"   Select MP3 output bitrate: [1] 192 kbps, [2] 256 kbps, [3] 320 kbps - ";
static char szOverwrite[]=	"\n ATTENTION: Overwrite previously created WAV and MP3 files ? [Y/N] - ";
static char szTopSpeedSel[]="\n NOTE: Not all MSX computers support 3600 baud cassette I/O speed. If MP3 file\n       playback causes \"Device I/O error\" on MSX, try to increase MP3 bitrate.\n       Also certain sound hardware may not play 3600 baud files correctly.\n";
static char szCreateOutFile[]=	"\n Creating output file \"%s\". Please wait...\n";
static char szOverwrWarning[]=	"\n WARNING: File with this name already exists. Overwrite ? [Y/N] - ";
static char szConvertToCD[]="\n Converting WAV file into 16 bit/stereo/44 kHz format...\n";
static char szNoLameFiles[]="\n WARNING: Lame encoder is not found, MP3 file will not be created...\n";
static char szCreateMP3[]=	" Creating MP3 output, this may take a while. Please wait...\n";
static char szWavCopyrMsg[]=" *** This WAV file was produced by CasLink2 *** ";
static char szCheckingIn[]=	" Checking input file format...\n";
static char szFiltering[] =	" Applying filter before decoding...\n";
static char szDetectSpeed[]=" Detecting MSX cassette interface speed in WAV file - ";
static char szSpeed1200[]=	"1200 bps detected!\n";
static char szSpeed2400[]=	"2400 bps detected!\n";
static char szSpeedError[]=	"FAILED!\n";
static char szProcessing[]=	"Decoding file contents, please wait...\n";
static char BatCmdLine[]=	"lame.exe --silent -q 0 -b %d -k -m m --cbr --noath --tt \"CasLink2: %s (MSX %d baud) %d kbps\" --tc \"This MP3 file was produced by CasLink2\" %s.wav %s.mp3";

// Error messages
static char	ErrorList[][80] = {
	" ERROR: Incorrect parameter passed to command line...\n\n",						// 1
	" ERROR: Insufficient command line information. Need more options...\n\n",			// 2
	" ERROR: Incorrect value specified...\n\n",											// 3
	" ERROR: Failed to allocate memory for output buffer...\n\n", 						// 4
	" ERROR: Failed to allocate memory for input buffer...\n\n", 						// 5
	" ERROR: Failed to open input file...\n\n", 										// 6
	" ERROR: Failed to manipulate input file pointer...\n\n", 							// 7
	" ERROR: Input file can't be processed...\n\n", 									// 8
	" ERROR: Failed to read input file...\n\n", 										// 9
	" ERROR: Input file format is not supported...\n\n", 								// 10
	" CANCEL: Overwrite request canceled by user...\n\n", 								// 11
	" ERROR: Failed to create output file...\n\n", 		 								// 12
	" ERROR: Failed to write to output file...\n\n", 		 							// 13
	" STOP: This type of 32kb ROM is not supported...\n\n", 	  						// 14
	" STOP: This type of file is not supported in multi-file mode...\n\n", 				// 15
	" STOP: Unknown file format: conversion is not possible...\n\n", 				 	// 16
	" STOP: Basic program can't be loaded after binary in multi-file mode...\n\n",		// 17
	" ERROR: Failed to convert WAV file...\n\n", 		 								// 18
	" ERROR: Failed to play the created WAV file...\n\n", 								// 19
	" ERROR: The input file is not a WAV file...\n\n",	 								// 20
	" ERROR: The input WAV file should be 16 bit/stereo/44 kHz...\n\n",	 				// 21
	" ERROR: Failed to convert to MP3 file...\n\n", 	 								// 22
	" ERROR: WAV file can not be processed...\n\n",										// 23
	" ERROR: Wrong CR+LF character sequence in the input ASCII file...\n\n",			// 24
	" ERROR: BIN file header contains conflicting information...\n\n",					// 25
	""
   };

// Subroutine declaration
int	WriteDataByte(unsigned char);
int	WriteServiceByte(unsigned char);
int WriteHeaderByte(unsigned char*);
int ConvertToCDQuality(void);
int ConvertToRDQuality(void);
void ExitOnError(int ErrorNum);
void CleanUp(void);
void DecodeFile(void);
void EncodeSingleFile(void);
void EncodeMultipleFiles(int);
void PrimarySetup(void);
void PlayOutputFile(void);
void ConvertOutputToMP3(void);
